// Shader-based Mob Spawn indicator by Rinvincibl3
// rinvincibl3@gmail.com
// initially inspired by VanillaTweaks
// Improves handling of blocks near chunk borders, partial blocks, and flowing lava

#version 150

#moj_import <fog.glsl>

uniform sampler2D Sampler0;

uniform vec4 ColorModulator;
uniform float FogStart;
uniform float FogEnd;
uniform vec4 FogColor;

in float vertexDistance;
in vec4 vertexColor;
in vec2 texCoord0;
in vec4 normal;
in vec2 lightLevel;
in vec2 faceCoords;
in float partialblock;
in float topblock;
in float lava;

out vec4 fragColor;

#define THICKNESS (1.5 / 16.0)
#define SQUARE (0)
#define FILLED_SQUARE (2)

bool isShape(int shape, vec2 uv, float scale){
    float cx = uv.x-0.5;
    float cy = uv.y-0.5;
    float width = 0.8*scale;
    float corner = -0.4;
    bool square  =  abs(cx + width/2)  < THICKNESS*width || 
                    abs(cy + width/2)  < THICKNESS*width ||
                    abs(cx - width/2)  < THICKNESS*width || 
                    abs(cy - width/2)  < THICKNESS*width;

    bool fsquare =  abs(cx)  < (width+THICKNESS)/2 && 
                    abs(cy)  < (width+THICKNESS)/2;

    bool corners =  abs(cx + cy) < width && abs(cx - cy) < width;

    if(shape == SQUARE){
        return (square && corners);
    } else if(shape == FILLED_SQUARE){
        return (fsquare && corners);
    } else {
        return true;
    }
}

vec4 plotShape(vec4 shapeColor, int shape, vec4 color, vec2 uv, float scale, float opacity){
    if (isShape(shape,uv,scale) ){
        return mix(color,shapeColor, opacity);
    }
    return color;
}


void main() {
    vec4 color = texture(Sampler0, texCoord0);
    if (color.a < 0.5) discard;

    vec4 WHITE  = vec4(1.0,1.0,1.0,1.0);
    vec4 RED    = vec4(2.0,0.5,0.5,1.0);
    vec4 ORANGE = vec4(1.0,0.8,0.5,1.0);
    vec4 YELLOW = vec4(1.0,1.0,0.0,1.0);
    vec4 LIGHT  = vec4(1.0,1.0,0.7,1.0);
    vec4 GREEN  = vec4(0.0,0.6,0.0,1.0);
    vec4 CYAN   = vec4(0.0,1.0,1.0,1.0);
    vec4 BLUE   = vec4(0.0,0.0,1.0,1.0);
    vec4 GRAY   = vec4(0.5,0.5,0.5,1.0);
    vec4 BLACK   = vec4(0.0,0.0,0.0,1.0);
    //Use the above color defaults to customize the color of the shapes

    vec4 vtc = vertexColor;

    if ((topblock > 0.01) && (partialblock < 0.01)&& (lava < 0.01)){
        float squaresize = 0.5; //affects size of the square overlays 1=full block
        float opacity = 0.2; //affects how strongly the overlay is shown
        if (lightLevel.x > 1.5*16+3) {
            color = plotShape(/*Overlay color: */ WHITE, /*Shape: */ FILLED_SQUARE, color, faceCoords, squaresize, opacity);
        } else if (lightLevel.x > 0.5*16+3) {
            color = plotShape(/*Overlay color: */ RED, /*Shape: */  SQUARE, color, faceCoords, squaresize, opacity);
        } else {
            color = plotShape(/*Overlay color: */ BLACK, /*Shape: */ SQUARE, color, faceCoords, squaresize, opacity/2);
        }
    }


    color *= vtc * ColorModulator;
    color = clamp(color, 0.0, 1.0);
    fragColor = linear_fog(color, vertexDistance, FogStart, FogEnd, FogColor);
}
