#version 150

#moj_import <light.glsl>
#moj_import <fog.glsl>

in vec3 Position;
in vec4 Color;
in vec2 UV0;
in ivec2 UV2;
in vec3 Normal;

uniform sampler2D Sampler2;

uniform mat4 ModelViewMat;
uniform mat4 ProjMat;
uniform vec3 ChunkOffset;
uniform int FogShape;

out float vertexDistance;
out vec4 vertexColor;
out vec2 texCoord0;
out vec4 normal;

out vec2 lightLevel;
out vec2 faceCoords;
out float partialblock;
out float topblock;
out float lava;

vec2 generateFaceCoords(){
    if (gl_VertexID % 4 == 0) return vec2(1.0,0.0);
    else if (gl_VertexID % 4 == 1) return vec2(1.0,1.0);
    else if (gl_VertexID % 4 == 2) return vec2(0.0,1.0);
    else if (gl_VertexID % 4 == 3) return vec2(0.0,0.0);
}

void main() {
    vec3 pos = Position + ChunkOffset;
    gl_Position = ProjMat * ModelViewMat * vec4(pos, 1.0);

    vertexDistance = fog_distance(ModelViewMat, pos, FogShape);
    vertexColor = Color * minecraft_sample_lightmap(Sampler2, UV2);
    texCoord0 = UV0;

    float fx = fract(Position.x);
    float fy = fract(Position.y);
    float fz = fract(Position.z);
    float t = 0.001;
    float yt = (1.0/8.0);

    partialblock = 0.0;
    if (( fx > t && fx < (1-t))|| 
        ( fz > t && fz < (1-t))|| 
        ((fy > t && fy < (1-t) && fy != yt))) {
        partialblock = 1.0;
    }
    topblock = 0.0;
    if ((Normal == vec3(0.0,1.0,0.0)) ){
        topblock = 1.0;
    }
    lava = 0.0;
    if ((UV2[0] > 14.999*16) ){
        lava = 1.0;
    }

    lightLevel = UV2;
    
    const vec2[4] corners = vec2[4](
        vec2(0, 1),
        vec2(0, 0),
        vec2(1, 0),
        vec2(1, 1)
    );

    normal = ProjMat * ModelViewMat * vec4(Normal, 0.0);
    faceCoords = corners[gl_VertexID % 4];
}
